/*****************************************************************************
* Copyright 2003 - 2009 Broadcom Corporation.  All rights reserved.
*
* Unless you and Broadcom execute a separate written software license
* agreement governing use of this software, this software is licensed to you
* under the terms of the GNU General Public License version 2, available at
* http://www.broadcom.com/licenses/GPLv2.php (the "GPL").
*
* Notwithstanding the above, under no circumstances may you combine this
* software in any way with any other Broadcom software provided under a
* license other than the GPL, without Broadcom's express prior written
* consent.
*****************************************************************************/

#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/ioport.h>
#include <linux/platform_device.h>
#include <linux/serial_8250.h>
#include <mach/hw_cfg.h>

#include <mach/hardware.h>
#include <mach/irqs.h>
#include <mach/pinmux.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/page.h>

#include <asm/mach/map.h>
#include <asm/pgalloc.h>

#include <linux/serial.h>
#include <linux/serial_core.h>
#include <plat/irvine.h>

#include <linux/vgpio.h>

#ifdef CONFIG_USB_GADGET_DWC_OTG
#include <asm/arch/lm.h>
//#include <asm/arch/reg_sys.h>
//#include <asm/arch/reg_usb.h>
//#include <asm/arch/reg_clkpwr.h>
#endif

#define IO_DESC(pa, sz) { .virtual = IO_ADDRESS(pa), \
                          .pfn = __phys_to_pfn(pa), \
                          .length = (sz), \
                          .type = MT_DEVICE }

static struct map_desc bcm476x_io_desc[] __initdata =
{
	IO_DESC(BCM47XX_SRAM_ADDRBASE, SZ_64K),
	IO_DESC(BCM47XX_NAND_ADDRBASE0,  SZ_64M),
	IO_DESC(BCM47XX_NAND_ADDRBASE1,  SZ_64M),

	IO_DESC(VIC0_REG_BASE_ADDR, SZ_4K),
	IO_DESC(VIC1_REG_BASE_ADDR, SZ_4K),
	IO_DESC(DMA_REG_BASE_ADDR, SZ_4K),
	IO_DESC(DDR_REG_BASE_ADDR, SZ_4K),
	IO_DESC(LCD_REG_BASE_ADDR, SZ_4K),
 IO_DESC(GFX_REG_BASE_ADDR, 4 * SZ_4K),
 IO_DESC(HPI_REG_BASE_ADDR, SZ_4K),
 IO_DESC(USB_REG_BASE_ADDR, SZ_4K),
 IO_DESC(SDM0_REG_BASE_ADDR, SZ_4K),
 IO_DESC(SDM1_REG_BASE_ADDR, SZ_4K),
 IO_DESC(SDM2_REG_BASE_ADDR, SZ_4K),
 IO_DESC(FLS_REG_BASE_ADDR, SZ_4K),
 IO_DESC(ETM_REG_BASE_ADDR, SZ_4K),
 IO_DESC(TIM1_REG_BASE_ADDR, SZ_4K),
 IO_DESC(RPC_REG_BASE_ADDR, SZ_4K),
 IO_DESC(GIO1_REG_BASE_ADDR, SZ_4K),
 IO_DESC(URT2_REG_BASE_ADDR, SZ_4K),
 IO_DESC(PWM_REG_BASE_ADDR, SZ_4K),
 IO_DESC(TSC_REG_BASE_ADDR, SZ_4K),
 IO_DESC(AUD0_REG_BASE_ADDR, SZ_4K),
 IO_DESC(AUD1_REG_BASE_ADDR, SZ_4K),
 IO_DESC(I2S_REG_BASE_ADDR, SZ_4K),
 IO_DESC(URT0_REG_BASE_ADDR, SZ_4K),
 IO_DESC(URT1_REG_BASE_ADDR, SZ_4K),
 IO_DESC(URTG_REG_BASE_ADDR, SZ_4K),
 IO_DESC(I2C0_REG_BASE_ADDR, SZ_4K),
 IO_DESC(I2C1_REG_BASE_ADDR, SZ_4K),
 IO_DESC(SPI0_REG_BASE_ADDR, SZ_4K),
 IO_DESC(SPI1_REG_BASE_ADDR, SZ_4K),
 IO_DESC(OTP_REG_BASE_ADDR, SZ_4K),
 IO_DESC(TIM0_REG_BASE_ADDR, SZ_4K),
 IO_DESC(GIO0_REG_BASE_ADDR, SZ_4K),
 IO_DESC(RTC_REG_BASE_ADDR, SZ_4K),
 IO_DESC(CMU_REG_BASE_ADDR, SZ_4K),
 IO_DESC(WDT_REG_BASE_ADDR, SZ_4K),
 IO_DESC(AXA_REG_BASE_ADDR, SZ_4K),
 IO_DESC(SPM_AHB_REG_BASE_ADDR, SZ_64K),
 IO_DESC(SPM_APB_REG_BASE_ADDR, SZ_4K),
 IO_DESC(PKA_REG_BASE_ADDR, SZ_4K),
 IO_DESC(RNG_REG_BASE_ADDR, SZ_4K),
 IO_DESC(PML_REG_BASE_ADDR, SZ_4K),
};

#if defined( CONFIG_SERIAL_8250 )

static struct uart_port bcm476x_serial_ports[] =
{
    {
        .membase	= (unsigned char *)(IO_ADDRESS(URT0_REG_BASE_ADDR)),
        .irq		= BCM4760_INTR_UART0,
        .flags		= UPF_BOOT_AUTOCONF,
        .iotype		= UPIO_MEM32,
        .regshift	= 2,
        .uartclk	= 0, /* computed at runtime */
        .line		= 0,
        .type		= PORT_UNKNOWN,
    },

    {
        .membase	= (unsigned char *)(IO_ADDRESS(URT1_REG_BASE_ADDR)),
        .irq		= BCM4760_INTR_UART1,
        .flags		= UPF_BOOT_AUTOCONF,
        .iotype		= UPIO_MEM32,
        .regshift	= 2,
        .uartclk	= 0, /* computed at runtime */
        .line		= 1,
        .type		= PORT_UNKNOWN,
        .fifosize	= 15
    },

    {
        .membase	= (unsigned char *)(IO_ADDRESS(URT2_REG_BASE_ADDR)),
        .irq		= BCM4760_INTR_UART2,
        .flags		= UPF_BOOT_AUTOCONF,
        .iotype		= UPIO_MEM32,
        .regshift	= 2,
        .uartclk	= 0, /* computed at runtime */
        .line		= 2,
        .type		= PORT_UNKNOWN,
    },

    {
        .membase	= (unsigned char *)(IO_ADDRESS(URTG_REG_BASE_ADDR)),
        .irq		= BCM4760_INTR_UART_GPS,
        .flags		= UPF_BOOT_AUTOCONF,
        .iotype		= UPIO_MEM32,
        .regshift	= 2,
        .uartclk	= 0, /* computed at runtime */
        .line		= 3,
        .type		= PORT_UNKNOWN,
    },
};
#endif // CONFIG_SERIAL_8250


//TODO: eventually, this function will be replaced by an equivalent opmode function...
static unsigned int get_pll_freq(uint32_t cmpll_phys_addr)
{
    uint32_t    cmpll = readl(IO_ADDRESS(cmpll_phys_addr));
    uint32_t    enab = (cmpll >> 18) & 0x1;
    uint32_t    pdiv, qdiv;
    unsigned long long freq;

    if( enab == 0 )
        return 0; /* pll is disabled */

    pdiv = cmpll & 0x1F;
    if( pdiv == 0 )
        pdiv = 32;

    qdiv = (cmpll >> 8) & 0x1FF;
    if( qdiv == 0 )
        qdiv = 512;

    freq = (24000000LL * qdiv);

    // This is a completely non-standard calling convention macro
    // provided by linux kernel.
    do_div(freq, pdiv);

    return freq;
}

//TODO: eventually, this function will be replaced by an equivalent opmode function...
static unsigned int get_uart_clk(uint32_t cmuart_phys_addr)
{
//#ifdef CONFIG_ARCH_BCM2730_FPGA || CONFIG_ARCH_BCM4760
#if 1
    return 24000000;
#else
    uint32_t cmuart = readl(IO_ADDRESS(cmuart_phys_addr));
    uint32_t csrc = (cmuart >> 8) & 0x7; // bit10:8
    uint32_t div = (cmuart & 0x1F);
    unsigned int freq;

    //get the source (bit10:8)
    switch(csrc)
    {
        case 1:
        freq = 24000000; /* using 24MHz oscillator */
        break;

        case 4:
        case 5:
        case 6:
        case 7:
        freq = get_pll_freq(BCM28XX_CMPLLS_PHY + (csrc-4)*4);
        break;

        default:
        return 0; /* invalid source */
    }

    if( div == 0 )
        div = 32;

    return (freq + div/2)/div;
#endif
}

void bcm47xx_pin_share_init (void)
{
}

//TODO: check if it's better to put in arch.c
#ifdef CONFIG_USB_GADGET_DWC_OTG

int __init usb_devices_init( void )
{
   struct lm_device *lmdev;
   int rc = -1;
/*   uint32_t mode_ctl = readl( IO_ADDRESS( CMU_R_MODE_CTL_MEMADDR ) ); */
   uint32_t pll_ctl = readl( IO_ADDRESS( CMU_R_PLA_PLL_CTL0_MEMADDR ) );

#if 1   // @KP: 090316: setup for usb device mode

   // bypass PMU for usb device testing; reg 0xb00a8
   // writel(0x6a3, IO_ADDRESS(CMU_R_MODE_CTL_MEMADDR));

   // config usb broadcom phy (8-bit) for otg mode (default to device mode); reset and unreset pair; reg 0xb0244
   writel(0x004069d5, IO_ADDRESS(CMU_R_USB_PHY0_MEMADDR));
   writel(0x0041a9d5, IO_ADDRESS(CMU_R_USB_PHY0_MEMADDR));

#else   // force host mode

   // bypass PMU for usb host testing; reg 0xb00a8
   writel(0x6aa3, IO_ADDRESS(CMU_R_MODE_CTL_MEMADDR));

   // config usb broadcom phy (8-bit) for host mode; reset and unreset pair; reg 0xb0244
   writel(0x004069c1, IO_ADDRESS(CMU_R_USB_PHY0_MEMADDR));
   writel(0x0041a9c1, IO_ADDRESS(CMU_R_USB_PHY0_MEMADDR));

#endif

#if 0   // @KP: 090825: testing for Otavalo which does not connect STAT1/2 from PMU to 4760

   mode_ctl&=~0x1FF80;
   writel( 0x6, IO_ADDRESS( 0xb0254 ) );
   writel( mode_ctl | (0x0d << 7), IO_ADDRESS( CMU_R_MODE_CTL_MEMADDR ) );

#endif

   // enable differential pll for usb; reg 0xb0040 bit 9
   pll_ctl |= CMU_F_PLA_EN_CMLBUF6_MASK;
   writel(pll_ctl, IO_ADDRESS(CMU_R_PLA_PLL_CTL0_MEMADDR));

   lmdev = kmalloc(sizeof(struct lm_device), GFP_KERNEL);
   if ( lmdev) {
      memset( lmdev, 0 , sizeof(struct lm_device));

      lmdev->resource.start= USB_REG_BASE_ADDR;
      lmdev->resource.end= lmdev->resource.start + SZ_4K -1;
      lmdev->resource.flags= IORESOURCE_MEM ;

      lmdev->irq= BCM4760_INTR_USB;
      lmdev->id= -2;

      rc = lm_device_register(lmdev);
      if (rc)
          printk(KERN_ERR "can't register BCM4760 OTG core device\n");

   }

   return rc;
}

subsys_initcall( usb_devices_init );

#endif // CONFIG_USB_GADGET_DWC_OTG

void __init bcm476x_map_io( void )
{
	iotable_init( bcm476x_io_desc, ARRAY_SIZE( bcm476x_io_desc ));
}
